import {
  Loader,
  Metadata,
  IDictionary,
  IEventParam,
  Config,
} from '@zeedhi/core';

import {
  Form,
  Select,
  Button,
} from '@zeedhi/common';

import { PmeMask } from '@/config';
  
export class Pme00708SubstituicaoDeProdutosController {
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public modalController = Loader.getInstance('ModalController');

  public crudController = Loader.getInstance('CrudController');

  public utilities = Loader.getInstance('Utilities');
  
  public reportController = Loader.getInstance('ReportController');
  
  public appController = Loader.getInstance('AppController');
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    row.PRODUCTMASK = this.utilities.formatReportMask(mask);
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');

    if (this.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      row.reportFormat = component.name;
      this.utilities.activeLoadingButton(component);
      this.reportController.generateReport('report/LogSubstituicaoProdutos', row, row.CD_REPORT_CODE).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }

  private formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    if (form.validate()) {
      if (this.utilities.dateIsValid(row.DTDE, row.DTATE)) {
        valid = true;
      } else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    }
    return valid;
  }
  
  public setFilterProduct({ component }: IEventParam<Select>) {
    const cdfilial = this.appController.getCdCentral();
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilial,
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
}
