import {
  Loader, Metadata, Config, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Form, CheckboxMultiple, Button, Select,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme03100EmissaoCardDiaController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');

  private impConfig = {
    IMPNDPCAL: 'N',
    IMPNUTR: 'N',
    IMPMSG: 'N',
  };

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
  
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterServ({ component }: IEventParam<Select>) {
    const cdtipoCard = Metadata.getInstance<Select>('TIPOCARD');
    const conditions = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: cdtipoCard.value ? cdtipoCard.value : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');
    if (this.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      else {
        row.reportFormat = component.name;
        this.utilities.activeLoadingButton(component);
        this.reportController.generateReport('report/emissaoCardDia', row, row.CD_REPORT_CODE).finally(() => {
          this.utilities.disableLoadingButton(component);
        });
      }
    }
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    const paramsCard = this.utilities.getParamsCard();
    row.NRCARDPME = paramsCard.NRCARDPME;
    row.IMPNDPCAL = this.impConfig.IMPNDPCAL;
    row.IMPNUTR = this.impConfig.IMPNUTR;
    row.IMPMSG = this.impConfig.IMPMSG;
    row.NMFILIAGRE = row.UNIDADE ? row.UNIDADE.NMFILIAL : null;
    row.CDFILIAL = row.UNIDADE ? row.UNIDADE.CDFILIAL : null;
    row.CDSERVFILI = row.SERVICO ? row.SERVICO : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.PRODUCTMASK = this.utilities.formatReportMask(mask);
  }

  private formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    const periodoDe = row.PERIODODE;
    const periodoAte = row.PERIODOATE;
    form.validate();
    if (form.validate()) {
      const dtDe = row.DTDE;
      const dtAte = row.DTATE;

      if (this.utilities.dateIsValid(periodoDe, periodoAte) && this.utilities.dateIsValid(dtDe, dtAte)) {
        if (this.utilities.dateIsValid(periodoDe, dtDe) && this.utilities.dateIsValid(dtAte, periodoAte)) valid = true;
        else this.utilities.showAlert('alertError', 'pme::dtCompreendidaPerCard', 'error');
      } else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    }
    return valid;
  }

  public setImpConfig({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = Array.isArray(component.value) ? component.value.join('_') : component.value;
      const consPrat = Metadata.getInstance<CheckboxMultiple>('CONSPRAT');
      const quebra = Metadata.getInstance<CheckboxMultiple>('QUEBRA');

      if (items.includes('IMPNDPCAL')) this.impConfig.IMPNDPCAL = 'S';
      else this.impConfig.IMPNDPCAL = 'N';

      if (items.includes('IMPNUTR')) {
        this.impConfig.IMPNUTR = 'S';
        consPrat.disabled = false;
      } else {
        this.impConfig.IMPNUTR = 'N';
        consPrat.disabled = true;
      }

      if (items.includes('IMPMSG')) {
        quebra.disabled = true;
        this.impConfig.IMPMSG = 'S';
      } else {
        quebra.disabled = false;
        this.impConfig.IMPMSG = 'N';
      }
    }
  }

  public openModalSearch() {
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD');
    if (tipoCard.value) {
      this.modalController.openModal();
      this.utilities.setCdTipoCard(tipoCard.value);
    } else {
      this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
    }
  }

  public async createPeriodoModal() {
    this.modalController.createModal('/modals/gridPeriodo');
  }

  public destroyPeriodoModal() {
    this.modalController.destroyModal();
  }
}
